@@ -4,23 +4,18 @@ import sys
 PY3 = sys.version_info[0] == 3
 JYTHON = sys.platform.startswith('java')
 
-if PY3:
+if PY3:  # pragma: no cover
     text_type = str
 else: # pragma: no cover
     # py2
     text_type = unicode
 
-def bytes_(s, encoding='latin-1', errors='strict'):
-    if isinstance(s, text_type):
-        s = s.encode(encoding, errors)
-    return s
-
 def text_(s):
-    if not isinstance(s, text_type):
+    if not isinstance(s, text_type):  # pragma: no cover
         s = s.decode('utf-8')
     return s
 
-if PY3:
+if PY3:  # pragma: no cover
     def native_(s, encoding='latin-1', errors='strict'):
         if isinstance(s, text_type):
             return s
@@ -31,7 +26,7 @@ else:  # pragma: no cover
             return s.encode(encoding, errors)
         return str(s)
 
-if PY3:
+if PY3:  # pragma: no cover
     from io import StringIO
 else: # pragma: no cover
     from io import BytesIO
@@ -43,7 +38,7 @@ else: # pragma: no cover
             super(StringIO, self).write(s)
 
 
-if PY3:
+if PY3:  # pragma: no cover
     def reraise(tp, value, tb=None):
         if value.__traceback__ is not tb: # pragma: no cover
             raise value.with_traceback(tb)
@@ -67,7 +62,7 @@ else: # pragma: no cover
 """)
 
 
-try:
+try:  # pragma: no cover
     from threading import get_ident as get_thread_ident
 except ImportError: # pragma: no cover
     # py2
