@@ -18,6 +18,7 @@ are associated with the right transaction.
 """
 import sys
 import threading
+import itertools
 
 from zope.interface import implementer
 
@@ -60,6 +61,9 @@ def _new_transaction(txn, synchs):
 
 @implementer(ITransactionManager)
 class TransactionManager(object):
+    """
+    Single-thread implementation of `~transaction.interfaces.ITransactionManager`.
+    """
 
     def __init__(self, explicit=False):
         self.explicit = explicit
@@ -67,7 +71,7 @@ class TransactionManager(object):
         self._synchs = WeakSet()
 
     def begin(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         if self._txn is not None:
             if self.explicit:
@@ -80,7 +84,7 @@ class TransactionManager(object):
     __enter__ = lambda self: self.begin()
 
     def get(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         if self._txn is None:
             if self.explicit:
@@ -94,44 +98,44 @@ class TransactionManager(object):
         self._txn = None
 
     def registerSynch(self, synch):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         self._synchs.add(synch)
         if self._txn is not None:
             synch.newTransaction(self._txn)
 
     def unregisterSynch(self, synch):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         self._synchs.remove(synch)
 
     def clearSynchs(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         self._synchs.clear()
 
     def registeredSynchs(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         return bool(self._synchs)
 
     def isDoomed(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         return self.get().isDoomed()
 
     def doom(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         return self.get().doom()
 
     def commit(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         return self.get().commit()
 
     def abort(self):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         return self.get().abort()
 
@@ -142,7 +146,7 @@ class TransactionManager(object):
             self.abort()
 
     def savepoint(self, optimistic=False):
-        """ See ITransactionManager.
+        """ See `~transaction.interfaces.ITransactionManager`.
         """
         return self.get().savepoint(optimistic)
 
@@ -177,7 +181,7 @@ class TransactionManager(object):
             return lambda func: self.run(func, tries)
 
         if tries <= 0:
-            raise ValueError("tries must be positive")
+            raise ValueError("tries must be > 0")
 
         # These are ordinarily native strings, but that's
         # not required. A callable class could override them
@@ -190,37 +194,100 @@ class TransactionManager(object):
         name = text_(name) if name else u''
         doc = text_(doc) if doc else u''
 
-        if name != u'_':
+        if name and name != u'_':
             if doc:
                 doc = name + u'\n\n' + doc
             else:
                 doc = name
 
-        for i in range(1, tries + 1):  # pragma: no branch
+        for try_no in itertools.count(1):
             txn = self.begin()
             if doc:
                 txn.note(doc)
-
             try:
                 result = func()
-                txn.commit()
-            except Exception as v:
-                if i == tries:
-                    raise # that was our last chance
-                retry = self._retryable(v.__class__, v)
-                txn.abort()
-                if not retry:
-                    raise
-            else:
+                self.commit()
                 return result
+            except BaseException as exc:
+                # Note: `abort` must not be called before `_retryable`
+                retry = (isinstance(exc, Exception)
+                         and try_no < tries
+                         and self._retryable(exc.__class__, exc))
+                self.abort()
+                if retry:
+                    continue
+                else:
+                    raise
 
 
-class ThreadTransactionManager(TransactionManager, threading.local):
-    """Thread-aware transaction manager.
+@implementer(ITransactionManager)
+class ThreadTransactionManager(threading.local):
+    """
+    Thread-local `transaction manager <transaction.interfaces.ITransactionManager>`.
 
-    Each thread is associated with a unique transaction.
+    A thread-local transaction manager can be used as a global
+    variable, but has a separate copy for each thread.
+
+    Advanced applications can use the `manager` attribute to get a
+    wrapped `TransactionManager` to allow cross-thread calls for
+    graceful shutdown of data managers.
     """
 
+    def __init__(self):
+        self.manager = TransactionManager()
+
+    @property
+    def explicit(self):
+        return self.manager.explicit
+
+    @explicit.setter
+    def explicit(self, v):
+        self.manager.explicit = v
+
+    def begin(self):
+        return self.manager.begin()
+
+    def get(self):
+        return self.manager.get()
+
+    def __enter__(self):
+        return self.manager.__enter__()
+
+    def commit(self):
+        return self.manager.commit()
+
+    def abort(self):
+        return self.manager.abort()
+
+    def __exit__(self, t, v, tb):
+        return self.manager.__exit__(t, v, tb)
+
+    def doom(self):
+        return self.manager.doom()
+
+    def isDoomed(self):
+        return self.manager.isDoomed()
+
+    def savepoint(self, optimistic=False):
+        return self.manager.savepoint(optimistic)
+
+    def registerSynch(self, synch):
+        return self.manager.registerSynch(synch)
+
+    def unregisterSynch(self, synch):
+        return self.manager.unregisterSynch(synch)
+
+    def clearSynchs(self):
+        return self.manager.clearSynchs()
+
+    def registeredSynchs(self):
+        return self.manager.registeredSynchs()
+
+    def attempts(self, number=3):
+        return self.manager.attempts(number)
+
+    def run(self, func=None, tries=3):
+        return self.manager.run(func, tries)
 
 class Attempt(object):
 
