@@ -11,7 +11,6 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ############################################################################
-import binascii
 import logging
 import sys
 import warnings
@@ -25,8 +24,6 @@ from transaction.interfaces import TransactionFailedError
 from transaction import interfaces
 from transaction._compat import reraise
 from transaction._compat import get_thread_ident
-from transaction._compat import native_
-from transaction._compat import bytes_
 from transaction._compat import StringIO
 from transaction._compat import text_type
 
@@ -45,12 +42,7 @@ def _makeLogger(): #pragma NO COVER
     return logging.getLogger("txn.%d" % get_thread_ident())
 
 
-# The point of this is to avoid hiding exceptions (which the builtin
-# hasattr() does).
-def myhasattr(obj, attr):
-    return getattr(obj, attr, _marker) is not _marker
-
-class Status:
+class Status(object):
     # ACTIVE is the initial state.
     ACTIVE       = "Active"
 
@@ -63,11 +55,17 @@ class Status:
     # to commit or join this transaction will raise TransactionFailedError.
     COMMITFAILED = "Commit failed"
 
-@implementer(interfaces.ITransaction,
-             interfaces.ITransactionDeprecated)
+class _NoSynchronizers(object):
+
+    @staticmethod
+    def map(_f):
+        "Does nothing"
+
+@implementer(interfaces.ITransaction)
 class Transaction(object):
-
-
+    """
+    Default implementation of `~transaction.interfaces.ITransaction`.
+    """
 
     # Assign an index to each savepoint so we can invalidate later savepoints
     # on rollback.  The first index assigned is 1, and it goes up by 1 each
@@ -79,7 +77,7 @@ class Transaction(object):
     _savepoint2index = None
 
     # Meta data. extended_info is also metadata, but is initialized to an
-    # emtpy dict in __init__.
+    # empty dict in __init__.
     _user = u""
     _description = u""
 
@@ -120,6 +118,12 @@ class Transaction(object):
         # List of (hook, args, kws) tuples added by addAfterCommitHook().
         self._after_commit = []
 
+        # List of (hook, args, kws) tuples added by addBeforeAbortHook().
+        self._before_abort = []
+
+        # List of (hook, args, kws) tuples added by addAfterAbortHook().
+        self._after_abort = []
+
     @property
     def _extension(self):
         # for backward compatibility, since most clients used this
@@ -136,8 +140,9 @@ class Transaction(object):
 
     @user.setter
     def user(self, v):
-        if v is not None:
-            self._user = text_or_warn(v)
+        if v is None:
+            raise ValueError("user must not be None")
+        self._user = text_or_warn(v)
 
     @property
     def description(self):
@@ -149,12 +154,12 @@ class Transaction(object):
             self._description = text_or_warn(v)
 
     def isDoomed(self):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         return self.status is Status.DOOMED
 
     def doom(self):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if self.status is not Status.DOOMED:
             if self.status is not Status.ACTIVE:
@@ -173,7 +178,7 @@ class Transaction(object):
                 self._failure_traceback.getvalue())
 
     def join(self, resource):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if self.status is Status.COMMITFAILED:
             self._prior_operation_failed() # doesn't return
@@ -184,12 +189,6 @@ class Transaction(object):
             # I think some users want it.
             raise ValueError("expected txn status %r or %r, but it's %r" % (
                              Status.ACTIVE, Status.DOOMED, self.status))
-        # TODO: the prepare check is a bit of a hack, perhaps it would
-        # be better to use interfaces.  If this is a ZODB4-style
-        # resource manager, it needs to be adapted, too.
-        if myhasattr(resource, "prepare"):
-            # TODO: deprecate 3.6
-            resource = DataManagerAdapter(resource)
         self._resources.append(resource)
 
         if self._savepoint2index:
@@ -216,7 +215,7 @@ class Transaction(object):
         self._resources = [r for r in self._resources if r is not resource]
 
     def savepoint(self, optimistic=False):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if self.status is Status.COMMITFAILED:
             self._prior_operation_failed() # doesn't return, it raises
@@ -252,35 +251,8 @@ class Transaction(object):
             savepoint.transaction = None # invalidate
         self._savepoint2index.clear()
 
-
-    def register(self, obj):
-        """ See ITransaction.
-        """
-        # The old way of registering transaction participants.
-        #
-        # register() is passed either a persisent object or a
-        # resource manager like the ones defined in ZODB.DB.
-        # If it is passed a persistent object, that object should
-        # be stored when the transaction commits.  For other
-        # objects, the object implements the standard two-phase
-        # commit protocol.
-        manager = getattr(obj, "_p_jar", obj)
-        if manager is None:
-            raise ValueError("Register with no manager")
-        adapter = self._adapters.get(manager)
-        if adapter is None:
-            adapter = MultiObjectResourceAdapter(manager)
-            adapter.objects.append(obj)
-            self._adapters[manager] = adapter
-            self.join(adapter)
-        else:
-            # TODO: comment out this expensive assert later
-            # Use id() to guard against proxies.
-            assert id(obj) not in map(id, adapter.objects)
-            adapter.objects.append(obj)
-
     def commit(self):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if self.status is Status.DOOMED:
             raise interfaces.DoomedTransaction(
@@ -311,9 +283,9 @@ class Transaction(object):
             finally:
                 del t, v, tb
         else:
-            self._free()
             self._synchronizers.map(lambda s: s.afterCompletion(self))
             self._callAfterCommitHooks(status=True)
+            self._free()
         self.log.debug("commit")
 
     def _saveAndGetCommitishError(self):
@@ -346,12 +318,12 @@ class Transaction(object):
             del t, v, tb
 
     def getBeforeCommitHooks(self):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         return iter(self._before_commit)
 
     def addBeforeCommitHook(self, hook, args=(), kws=None):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if kws is None:
             kws = {}
@@ -359,54 +331,101 @@ class Transaction(object):
 
     def _callBeforeCommitHooks(self):
         # Call all hooks registered, allowing further registrations
-        # during processing.  Note that calls to addBeforeCommitHook() may
-        # add additional hooks while hooks are running, and iterating over a
-        # growing list is well-defined in Python.
-        for hook, args, kws in self._before_commit:
-            hook(*args, **kws)
-        self._before_commit = []
+        # during processing.
+        self._call_hooks(self._before_commit)
 
     def getAfterCommitHooks(self):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         return iter(self._after_commit)
 
     def addAfterCommitHook(self, hook, args=(), kws=None):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if kws is None:
             kws = {}
         self._after_commit.append((hook, tuple(args), kws))
 
     def _callAfterCommitHooks(self, status=True):
-        # Avoid to abort anything at the end if no hooks are registred.
-        if not self._after_commit:
+        self._call_hooks(self._after_commit,
+                         exc=False, clean=True, prefix_args=(status,))
+
+    def _call_hooks(self, hooks, exc=True, clean=False, prefix_args=()):
+        """call *hooks*.
+
+        If *exc* is true, fail on the first exception; otherwise
+        log the exception and continue.
+
+        If *clean* is true, abort all resources. This is to ensure
+        a clean state should a (after) hook has affected one
+        of the resources.
+
+        *prefix_args* defines additional arguments prefixed
+        to the arguments provided by the hook definition.
+
+        ``_call_hooks`` supports that a hook adds new hooks.
+        """
+        # Avoid to abort anything at the end if no hooks are registered.
+        if not hooks:
             return
+        try:
+            # Call all hooks registered, allowing further registrations
+            # during processing
+            for hook, args, kws in hooks:
+                try:
+                    hook(*(prefix_args + args), **kws)
+                except:
+                    if exc:
+                        raise
+                    # We should not fail
+                    self.log.error("Error in hook exec in %s ",
+                                   hook, exc_info=sys.exc_info())
+        finally:
+            del hooks[:]  # clear hooks
+            if not clean:
+                return
+            # The primary operation has already been performed.
+            # But the hooks execution might have left the resources
+            # in an unclean state. Clean up
+            for rm in self._resources:
+                try:
+                    rm.abort(self)
+                except:
+                    # XXX should we take further actions here ?
+                    self.log.error("Error in abort() on manager %s",
+                                   rm, exc_info=sys.exc_info())
+
+    def getBeforeAbortHooks(self):
+        """ See `~transaction.interfaces.ITransaction`.
+        """
+        return iter(self._before_abort)
+
+    def addBeforeAbortHook(self, hook, args=(), kws=None):
+        """ See `~transaction.interfaces.ITransaction`.
+        """
+        if kws is None:
+            kws = {}
+        self._before_abort.append((hook, tuple(args), kws))
+
+    def _callBeforeAbortHooks(self):
         # Call all hooks registered, allowing further registrations
-        # during processing.  Note that calls to addAterCommitHook() may
-        # add additional hooks while hooks are running, and iterating over a
-        # growing list is well-defined in Python.
-        for hook, args, kws in self._after_commit:
-            # The first argument passed to the hook is a Boolean value,
-            # true if the commit succeeded, or false if the commit aborted.
-            try:
-                hook(status, *args, **kws)
-            except:
-                # We need to catch the exceptions if we want all hooks
-                # to be called
-                self.log.error("Error in after commit hook exec in %s ",
-                               hook, exc_info=sys.exc_info())
-        # The transaction is already committed. It must not have
-        # further effects after the commit.
-        for rm in self._resources:
-            try:
-                rm.abort(self)
-            except:
-                # XXX should we take further actions here ?
-                self.log.error("Error in abort() on manager %s",
-                               rm, exc_info=sys.exc_info())
-        self._after_commit = []
-        self._before_commit = []
+        # during processing.
+        self._call_hooks(self._before_abort, exc=False)
+
+    def getAfterAbortHooks(self):
+        """ See `~transaction.interfaces.ITransaction`.
+        """
+        return iter(self._after_abort)
+
+    def addAfterAbortHook(self, hook, args=(), kws=None):
+        """ See `~transaction.interfaces.ITransaction`.
+        """
+        if kws is None:
+            kws = {}
+        self._after_abort.append((hook, tuple(args), kws))
+
+    def _callAfterAbortHooks(self):
+        self._call_hooks(self._after_abort, clean=True)
 
     def _commitResources(self):
         # Execute the two-phase commit protocol.
@@ -462,20 +481,42 @@ class Transaction(object):
             except Exception:
                 self.log.error("Error in tpc_abort() on manager %s",
                                rm, exc_info=sys.exc_info())
+    def _free_manager(self):
+        try:
+            if self._manager:
+                self._manager.free(self)
+        finally:
+            # If we try to abort a transaction and fail, the manager
+            # may have begun a new transaction, and will raise a
+            # ValueError from free(); we don't want that to happen
+            # again in _free(), which abort() always calls, so be sure
+            # to clear out the manager.
+            self._manager = None
 
     def _free(self):
         # Called when the transaction has been committed or aborted
         # to break references---this transaction object will not be returned
         # as the current transaction from its manager after this, and all
         # IDatamanager objects joined to it will forgotten
-        if self._manager:
-            self._manager.free(self)
+        # All hooks and data are forgotten.
+        self._free_manager()
 
         if hasattr(self, '_data'):
             delattr(self, '_data')
 
         del self._resources[:]
 
+        del self._before_commit[:]
+        del self._after_commit[:]
+        del self._before_abort[:]
+        del self._after_abort[:]
+
+        # self._synchronizers might be shared, we can't mutate it
+        self._synchronizers = _NoSynchronizers
+        self._adapters = None
+        self._voted = None
+        self.extension = None
+
     def data(self, ob):
         try:
             data = self._data
@@ -496,19 +537,24 @@ class Transaction(object):
         data[id(ob)] = ob_data
 
     def abort(self):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
-        if self._savepoint2index:
-            self._invalidate_all_savepoints()
-
-        self._synchronizers.map(lambda s: s.beforeCompletion(self))
-
         try:
-
             t = None
             v = None
             tb = None
 
+            self._callBeforeAbortHooks()
+            if self._savepoint2index:
+                self._invalidate_all_savepoints()
+
+            try:
+                self._synchronizers.map(lambda s: s.beforeCompletion(self))
+            except:
+                t, v, tb = sys.exc_info()
+                self.log.error("Failed to call synchronizers", exc_info=sys.exc_info())
+
+
             for rm in self._resources:
                 try:
                     rm.abort(self)
@@ -518,7 +564,12 @@ class Transaction(object):
                     self.log.error("Failed to abort resource manager: %s",
                                    rm, exc_info=sys.exc_info())
 
-            self._free()
+
+            self._callAfterAbortHooks()
+            # Unlike in commit(), we are no longer the current transaction
+            # when we call afterCompletion(). But we can't be completely _free():
+            # the synchronizer might want to access some data it set before.
+            self._free_manager()
 
             self._synchronizers.map(lambda s: s.afterCompletion(self))
 
@@ -527,10 +578,11 @@ class Transaction(object):
             if tb is not None:
                 reraise(t, v, tb)
         finally:
+            self._free()
             del t, v, tb
 
     def note(self, text):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         if text is not None:
             text = text_or_warn(text).strip()
@@ -540,159 +592,31 @@ class Transaction(object):
                 self.description = text
 
     def setUser(self, user_name, path=u"/"):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         self.user = u"%s %s" % (text_or_warn(path), text_or_warn(user_name))
 
     def setExtendedInfo(self, name, value):
-        """ See ITransaction.
+        """ See `~transaction.interfaces.ITransaction`.
         """
         self.extension[name] = value
 
+    def isRetryableError(self, error):
+        return self._manager._retryable(type(error), error)
+
 
 # TODO: We need a better name for the adapters.
 
 
-class MultiObjectResourceAdapter(object):
-    """Adapt the old-style register() call to the new-style join().
-
-    With join(), a resource manager like a Connection registers with
-    the transaction manager.  With register(), an individual object
-    is passed to register().
-    """
-    def __init__(self, jar):
-        self.manager = jar
-        self.objects = []
-        self.ncommitted = 0
-
-    def __repr__(self):
-        return "<%s for %s at %s>" % (self.__class__.__name__,
-                                      self.manager, id(self))
-
-    def sortKey(self):
-        return self.manager.sortKey()
-
-    def tpc_begin(self, txn):
-        self.manager.tpc_begin(txn)
-
-    def tpc_finish(self, txn):
-        self.manager.tpc_finish(txn)
-
-    def tpc_abort(self, txn):
-        self.manager.tpc_abort(txn)
-
-    def commit(self, txn):
-        for o in self.objects:
-            self.manager.commit(o, txn)
-            self.ncommitted += 1
-
-    def tpc_vote(self, txn):
-        self.manager.tpc_vote(txn)
-
-    def abort(self, txn):
-        t = None
-        v = None
-        tb = None
-        try:
-            for o in self.objects:
-                try:
-                    self.manager.abort(o, txn)
-                except:
-                    # Capture the first exception and re-raise it after
-                    # aborting all the other objects.
-                    if tb is None:
-                        t, v, tb = sys.exc_info()
-                    txn.log.error("Failed to abort object: %s",
-                                  object_hint(o), exc_info=sys.exc_info())
-
-            if tb is not None:
-                reraise(t, v, tb)
-        finally:
-            del t, v, tb
-
-
 def rm_key(rm):
     func = getattr(rm, 'sortKey', None)
     if func is not None:
         return func()
 
-def object_hint(o):
-    """Return a string describing the object.
-
-    This function does not raise an exception.
-    """
-    # We should always be able to get __class__.
-    klass = o.__class__.__name__
-    # oid would be great, but maybe this isn't a persistent object.
-    oid = getattr(o, "_p_oid", _marker)
-    if oid is not _marker:
-        oid = oid_repr(oid)
-    else:
-        oid = 'None'
-    return "%s oid=%s" % (klass, oid)
-
-def oid_repr(oid):
-    if isinstance(oid, str) and len(oid) == 8:
-        # Convert to hex and strip leading zeroes.
-        as_hex = native_(
-            binascii.hexlify(bytes_(oid, 'ascii')), 'ascii').lstrip('0')
-        # Ensure two characters per input byte.
-        if len(as_hex) & 1:
-            as_hex = '0' + as_hex
-        elif as_hex == '':
-            as_hex = '00'
-        return '0x' + as_hex
-    else:
-        return repr(oid)
-
-
-# TODO: deprecate for 3.6.
-class DataManagerAdapter(object):
-    """Adapt zodb 4-style data managers to zodb3 style
-
-    Adapt transaction.interfaces.IDataManager to
-    ZODB.interfaces.IPureDatamanager
-    """
-
-    # Note that it is pretty important that this does not have a _p_jar
-    # attribute. This object will be registered with a zodb3 TM, which
-    # will then try to get a _p_jar from it, using it as the default.
-    # (Objects without a _p_jar are their own data managers.)
-
-    def __init__(self, datamanager):
-        self._datamanager = datamanager
-
-    # TODO: I'm not sure why commit() doesn't do anything
-
-    def commit(self, transaction):
-        # We don't do anything here because ZODB4-style data managers
-        # didn't have a separate commit step
-        pass
-
-    def abort(self, transaction):
-        self._datamanager.abort(transaction)
-
-    def tpc_begin(self, transaction):
-        # We don't do anything here because ZODB4-style data managers
-        # didn't have a separate tpc_begin step
-        pass
-
-    def tpc_abort(self, transaction):
-        self._datamanager.abort(transaction)
-
-    def tpc_finish(self, transaction):
-        self._datamanager.commit(transaction)
-
-    def tpc_vote(self, transaction):
-        self._datamanager.prepare(transaction)
-
-    def sortKey(self):
-        return self._datamanager.sortKey()
-
 
 @implementer(interfaces.ISavepoint)
-class Savepoint:
-    """Transaction savepoint.
+class Savepoint(object):
+    """Implementation of `~transaction.interfaces.ISavepoint`, a transaction savepoint.
 
     Transaction savepoints coordinate savepoints for data managers
     participating in a transaction.
@@ -719,7 +643,7 @@ class Savepoint:
         return self.transaction is not None
 
     def rollback(self):
-        """ See ISavepoint.
+        """ See `~transaction.interfaces.ISavepoint`.
         """
         transaction = self.transaction
         if transaction is None:
@@ -735,7 +659,7 @@ class Savepoint:
             transaction._saveAndRaiseCommitishError() # reraises!
 
 
-class AbortSavepoint:
+class AbortSavepoint(object):
 
     def __init__(self, datamanager, transaction):
         self.datamanager = datamanager
@@ -746,7 +670,7 @@ class AbortSavepoint:
         self.transaction._unjoin(self.datamanager)
 
 
-class NoRollbackSavepoint:
+class NoRollbackSavepoint(object):
 
     def __init__(self, datamanager):
         self.datamanager = datamanager
